@@ -130,12 +130,12 @@ class LeagueEntryDTO(RiotApiResponse):
                  hotStreak: bool, veteran: bool, freshBlood: bool, inactive: bool, miniSeries: Optional[dict] = None,
                  leagueId: Optional[str] = None, tier: Optional[str] = None, rank: Optional[str] = None):
         super().__init__()
-        self.leagueId: str = leagueId or '?'
+        self.leagueId = leagueId
         self.summonerId = summonerId
         self.summonerName = summonerName
         self.queueType = queueType
-        self.tier: str = tier or '?'
-        self.rank: str = rank or '?'
+        self.tier = tier
+        self.rank = rank
         self.leaguePoints = leaguePoints
         self.wins = wins
         self.losses = losses
@@ -150,7 +150,7 @@ class LeagueEntryDTO(RiotApiResponse):
     def __get_short(tier: str, rank: str) -> str:
         if not (tier and rank):
             return '??'
-        return f"{('GM' if tier.startswith('GR') else tier[0])}{'4' if rank.lower().endswith('v') else len(rank)}"
+        return f"{('GM' if tier.startswith('GR') else tier[0])}{'4' if rank.lower() == 'iv' else len(rank)}"
 
 
 class MiniSeriesDTO(RiotApiResponse):
